/******************************************
* config.h - Hard coded configuration info
*
* CVS $Id: config.h,v 1.19 2001/03/18 18:27:57 wcampbel Exp $
******************************************/

#ifndef CONFIG_H
#define CONFIG_H

/* Makefile and -D's were getting too much.  It's 
** a little more work to set a service up
** but it's vastly more configurable
*/

/* Many of the options from the Makefile are now here */

/* CRYPT stays in Makefile because of the possibility of a
** system without libcrypt, plus we need different libraries
** on different systems
*/

/* Solaris needs special options
** Win32 needs special options too
** They will PROBABLY remain in Makefile, at least for now
** as both systems require different CFLAGS and LDFLAGS
*/

/* Support fnmatch() - This is used to support masked servers
** eg *.fi *.au *.test.net.  This makes it non portable to some
** systems, Windows Cygwin included
** MOST networks do not need this, so it's disabled by default
** With it disabled, introducing a user on a server behind a mask
** will cause Stats to die.  The message when it dies will tell you
** to enable this option.  If you think that you will run a server
** with a mask, enable it now to prevent your service from dying
** NOTE:  It no longer crashes.  It sends out a message via WALLOPS
**        or GLOBOPS, logs the message in the log files, and gracefully
**        exits the service.
*/

#undef FNMATCH

/* Enable the more secure method of communicating with the service, 
** PRIVMSG user@server :message.  This is quite stable and usable, and is
** now enabled by default.  It does SOMEWHAT increase CPU utilization, but
** not by much
*/

#define SUPPORT_MSG_UH

/* Only allow the secure method of communicating with the service, disallowing
** PRIVMSG nick :message.
*/

#undef DISALLOW_NICK_PRIVMSG

/* This is another new feature... This is for support of the Hybrid 6
** NOTICE/PRIVMSG opers@server feature.  If this is enabled, all messages
** to opers@<my name> will be forwarded on to all logged in users.
** This does increase CPU usage so it is disabled by default
*/

#undef SUPPORT_MSG_OPERS

/* Define this to have the service ignore illegal top level domains
** This is useful on servers that allow spoofs for either opers 
** or users.  Currently, this will just prevent logging of the illegal
** TLD's to the log file.  If there is a valid ICANN approved TLD that is
** not working, please let me know so I can fix it.  
**
** NOTE:  I am not putting the Open Root Servers or the ADNS TLD's into
**        the official distribution.  If you use one or the other and
**        wish to have support, then you will have to create a TLD
**        table.  I could then distribute this TLD table along side the
**        official one.
**
** NOTE:  Your statistics are not changed by this option.  Any unknown
**        tld users are assigned to the 'unknown' TLD, and properly
**        shown in TLDMAP
*/

#undef IGNORE_ILLEGAL_TLD

/* If you wish the Stats service to track server VERSION's, and report them
** on a SERVER request and on the HTML dump, define this.  It will increase
** memory usage, CPU, and bandwidth on the network.  It won't be terribly
** noticeable though, and may be worth the extra
*/

#undef TRACK_VERSION

/* Disabling LAG_CHECK will remove Stats ability to track lag between
** servers on the network.  Disabling it will reduce CPU and Bandwidth
** consumption, but at a cost, less useful information stored by the 
** service.  Prior to this release, Stats ALWAYS sent the PING's out
** to the network.
*/

#define LAG_CHECK

/* On Dal derived servers, store the oper flags, and print the highest
** 'ranking' one out when OPERLIST is called.  Idea from Xride's ModeServ
** If well liked, it may be always enabled on Dal derived servers.
*/

#define OPER_MODES

/* If you dislike the service being able to JUPE, and were unhappy with
** JUPE being controlled by an entry in the config file, this is for you...
** JUPE must be #define'ed here to even exist in the service.
*/

#undef ENABLE_JUPE

/* Define this to allow any oper to issue the JUPE command.  With it undef'ed
** it will only allow people both opered and logged in to the service
**
** This used to be defined by default (in the GeoStats code).  Now it is
** optionally definable.  
*/

#undef ALL_OPER_JUPE

/* This will enable or disable QUOTE/RAW, just as dangerous
** #define it to have it enabled
*/

#undef ENABLE_QUOTE

/* Enable this option to create a 3/4 second artificial delay between
** sequential PRIVMSG's.  Note that this can and will lag your service.
** This option is ONLY in the conf to support servers with extremely
** strict flood control, enough to rate limit remote users... (Hyb7)
**
** NOTE:  This option shouldn't be necessary after Hybrid-7 beta2
*/

#undef DELAY_PRIVMSG

/* Large network support - The targeted audience of this package
** are small networks (Under 1000 users and 60 servers)
** Enabling this option will enlarge the data tables (and increase
** memory usage).  This is untested, and therefore may cause problems
** I would avoid enabling this until there is further testing
**
** This service will need to be heavily optimized to support a large
** number of users.  The optimization HAS begun, but is far from complete.
*/

#undef LARGE_NET

/* You MUST set this or RESTART won't work!!!!
** Set this to the path to the stats binary
*/

#define SPATH "/home/user/Stats/stats"

/* I hate having to do this... now the IRCD_TYPE is set at compile
** time.  This means to change the IRCD_TYPE, you must recompile the 
** service
**
** Only enable ONE - There is no checking or sanity checking on this 
** option yet...
**
** Supported types...
** IRCD_HYBRID
** IRCD_HYB6
** IRCD_HYB7
** IRCD_OPN
** IRCD_DAL
** IRCD_DALNQ
** IRCD_MLN
** IRCD_2_10
**
** Look at the README file for more detailed descriptions of these IRCD
** types.
*/
#define IRCD_HYBRID
/* #define IRCD_HYB6 */
/* #define IRCD_HYB7 */
/* #define IRCD_OPN */
/* #define IRCD_DAL */
/* #define IRCD_MLN */
/* #define IRCD_2_10 */
/* #define IRCD_DALNQ */

#endif /* CONFIG_H */
