/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: main.c,v 1.16 2001/04/08 14:04:13 wcampbel Exp $
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
** See the file called 'LICENSE' for more details.
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "patchlevel.h"

char s_StatServ[MAXNICK + 1] = "StatServ";
char s_Spam[8] = "cm-i";
int servsock;
const char version_date[] = __DATE__;
const char version_time[] = __TIME__;

#ifdef SUPPORT_MSG_UH
#if defined(IRCD_DAL) || defined(IRCD_DALNQ) || defined(IRCD_MLN)
char s_UH[MAXNICK + MAXHOST + 2];
#else
char s_UH[MAXUSER + MAXHOST + 2];
#endif
#endif

#ifdef SUPPORT_MSG_OPERS
/* 7 is "opers@" and the \0 terminator
** I should PROBABLY force this to only be enabled on Hybrid 6 
*/
char s_OPERS[MAXHOST + 7];
#endif

/* NOTE:  This code needs profiled to find out what needs to be optimized
** to make the code as efficient as possible.
*/
static void start();
static void CmdLine(int, char **);
static void PrintVersion();
static void PrintHelp();


int main(int argc, char *argv[])
{
	int forked = 0;
	FILE *fp;
	
	me.cmdflags = 0x00000000;

	CmdLine(argc, argv);

	log("Statistics Started (%s(%s) %s).", PATCHLEVEL, SERIALNUM,
		VersionFlags());
	me.t_start = time(NULL);

	InitDefaults();
	ConfLoad();
	me.chan_count = GetChanList();

	/* Sanity checking for FLOOD */
	if (me.flood == 1) {
		me.flood = 2;
	}

#ifdef SUPPORT_MSG_UH
	/* Create the user@host that we are to save some cycles later...
	** IF this features is used often
	*/
#if defined(IRCD_DAL) || defined(IRCD_DALNQ) || defined(IRCD_MLN)
	/* sigh... Dreamforge derived servers violate the RFC...
	** use nick@server instead of username@server (username@server
	** is REALLY a shortcut for username%hostname@server, which
	** *suprise* dreamforge doesn't support...
	*/
	snprintf(s_UH, (MAXHOST + MAXNICK + 2), "%s@%s", s_StatServ, me.name);
#else
	snprintf(s_UH, (MAXHOST + MAXUSER + 2), "%s@%s", StatServ.user,
		me.name);
#endif
#endif

#ifdef SUPPORT_MSG_OPERS
	/* Same as above...If we call it once, we'll definately call it 
	** again...
	*/
	snprintf(s_OPERS, (MAXHOST + 7), "opers@%s", me.name);
#endif

	/* This should ONLY be run once */
	srandom(time(NULL)); 

	LoadStats();
	LoadTLD();
	LoadMyUsers();

	if (!IsDebug() && !IsNoFork())
		forked=fork();
	if (forked) {
		fp = fopen(PIDFILE, "w");
		fprintf(fp, "%i", forked);
		fclose(fp);
		return 0;
	}

#if 0
	fclose(stdout);
	fclose(stdin);
	fclose(stderr);
#endif

	start();

	return 1;
}

void start()
{
	TimerReset();
	me.users = me.opers = me.servers = 0;
	init_server_hash();
	init_user_hash();
	init_tld();

	log("Connecting to %s:%d", me.uplink, me.port);
	if (servsock > 0)
		close(servsock);

	servsock = ConnectTo(me.uplink, me.port);

	if (servsock <= 0) {
		log("Unable to connect to %s port %d", me.uplink, me.port);
	} else {
		login();
		read_loop();
	}
	log("Reconnecting to the server in %d seconds", me.r_time);
	/* Here I need to flush the server/user table */
	if (IsDebug()) {
		if (!me.s) log("Invalid me.s");
	}
	if (me.s) FlushServer(me.s->name);
	sleep(me.r_time);
	start();
}

/* This is ugly, logging into each server type is slightly different */
void login()
{
#if defined(IRCD_HYBRID)
	sts("PASS %s :TS", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
	sts("PASS %s", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
#else
#if defined(IRCD_HYB6)
	sts("PASS %s :TS", me.pass);
	sts("CAPAB :QS");
	sts("SERVER %s 1 :%s", me.name, me.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_OPN)
	sts("PASS %s :TS", me.pass);
        sts("CAPAB :QS OPN");
        sts("SERVER %s 1 :%s", me.name, me.desc);
        sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_HYB7)
	/* NOTE:  a Stats service CANNOT be a LazyLink!!!!!!!!!
	** It's impossible for a Stats service to keep accurate
	** counts on anything if if doesn't receive a complete burst
	*/
	sts("PASS %s :TS", me.pass);
	/* Make sure to specify it can be a hub - This will only show up in
	** the STATS ? CAPAB list for the Stats service if it is H: lined
	** Always specifying HUB won't hurt anything though
        */
	sts("CAPAB :QS HUB");
        sts("SERVER %s 1 :%s", me.name, me.desc);
        sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_2_10)
	/* I currently don't send the "version flags" or that P strict crap */
	sts("PASS %s :0210", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
#else
#if defined(IRCD_DALNQ)
	/* Does PROTOCTL come before or after SERVER? */
	sts("PASS %s", me.pass);
	sts("PROTOCTL NOQUIT");
	sts("SERVER %s 1 :%s", me.name, me.desc);
#endif
#endif
#endif
#endif
#endif
#endif
#endif
}

/* Init the StatServ client */
void init_statserv()
{
	char mbuf[MAXUSER];
	char rname[LINE];

	snprintf(mbuf, MAXUSER, "+%s%s", IsSetOper() ? "o" : "", 
		IsSetDeaf() ? "d" : "");

#ifdef SUPPORT_MSG_UH
	snprintf(rname, LINE, "/msg %s HELP", s_UH);
#else
	snprintf(rname, LINE, "/msg %s HELP", s_StatServ);
#endif

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
	sts("NICK %s 1 1 %s %s %s %s :%s", s_StatServ,
		mbuf, StatServ.user, StatServ.host, me.name, rname);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN) || defined(IRCD_DALNQ)
	sts("NICK %s 1 1 %s %s %s 0 :%s", s_StatServ,
		StatServ.user, StatServ.host, me.name, rname);
	/* If it's anything EXCEPT just a plain "+", send the mode line */
	if (strcmp(mbuf, "+")) 
		sts(":%s MODE %s %s", s_StatServ, s_StatServ, mbuf);
#else
#if defined(IRCD_2_10)
	sts("NICK %s 1 %s %s %d %s :%s", s_StatServ,
		StatServ.user, StatServ.host, 
		/* This is the server token (which is ugly as sin) 
		** For now, I'll send '1'... Once I find out more about
		** it, I'll send the proper value.  
		*/
		1, mbuf, rname);
#endif
#endif
#endif

#ifdef IRCD_2_10
	AddUser(s_StatServ, StatServ.user, StatServ.host, findserver(me.name));
#else
	AddUser(s_StatServ, StatServ.user, StatServ.host, me.name);
#endif

/* If the StatServ client is an oper, increase oper count here
** and set it as an oper
*/
	if (IsSetOper()) {
		User* u = finduser(s_StatServ);
		IncreaseOpers(findserver(me.name));
		u->flags |= UFLAG_ISOPER;
	}
}

/* Bring up the Spam detecting client if it was enabled */
void init_spam()
{
	if (!IsSpam())
		return;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
	sts("NICK %s 1 1 + please %s %s :Spam Me!", s_Spam,
		me.name, me.name);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN) || defined(IRCD_DALNQ)
	sts("NICK %s 1 1 please %s %s :Spam Me!", s_Spam,
		me.name, me.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 please %s %d + :Spam Me!", s_Spam,
                me.name, 1);
                /* This is the server token (which is ugly as sin)
                ** For now, I'll send '1'... Once I find out more about
                ** it, I'll send the proper value.
                */
#endif
#endif
#endif

#ifdef IRCD_2_10
	AddUser(s_Spam, "please", me.name, findserver(me.name));
#else
	AddUser(s_Spam, "please", me.name, me.name);
#endif
}

/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Stats
*/
void init_join()
{
   int i;
   if (me.chan_count) {
	for (i = 0; i < me.chan_count; i++) {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
			sts(":%s SJOIN %ld %s + :@%s", me.name, 
			    time(NULL), me.chan_list[i], s_StatServ);
#else
#if defined(IRCD_DAL) || defined(IRCD_DALNQ)
			sts(":%s JOIN %s", s_StatServ, me.chan_list[i]);
#else
#if defined(IRCD_MLN)
			sts(":%s JOIN %s", s_StatServ, me.chan_list[i]);
			sts(":%s MODE %s +o %s %ld", me.name, me.chan_list[i],
			s_StatServ, time(NULL));
#else
#if defined(IRCD_2_10)
			/* Here is the only one that we hack ops... */
			sts(":%s NJOIN %s :@%s", me.name, me.chan_list[i], 
				s_StatServ);
#endif
#endif
#endif
#endif
	}
   }
}

void CmdLine(int argc, char *argv[])
{
	char c;
	extern char *optarg;
	extern int optind;

	while( (c=getopt(argc, argv, "dvhn")) != -1)
	{
		switch (c)
		{
			case 'd':
				me.cmdflags |= CMD_DEBUG;
				break;
			case 'v':
				PrintVersion();
				exit(0);
				/* NOT REACHED */
				break;
			case 'h':
				PrintHelp();
				exit(0);
				/* NOT REACHED */
				break;
			case 'n':
				me.cmdflags |= CMD_NOFORK;
				break;
		}
	}

	
}

void PrintVersion()
{
	printf("Stats version %s(%s)\nSPATH: %s\n", PATCHLEVEL, SERIALNUM,
		SPATH);
	printf("Stats NOT Started.\n");
}

void PrintHelp()
{
	printf("stats [-n|-d|-h|-v]\n-n Do not fork into the background\n");
	printf("-d Set debugging mode\n-h Print this help screen\n");
	printf("-v Print the version and exit\n");
	printf("Stats NOT Started.\n");
}
