/* parser.y - Conf file parser for Stats
** Copyright Stats coding team (2001).
**
** Based on code from ircd-hybrid-7
*/

%{

#include "stats.h"
#include "struct.h"
#include "proto.h"

int yyparse();

%}

%union {
	int number;
	char *string;
}

%token T_SERVER_NAME
%token T_SERVER_PORT
%token T_CONNECT_TO
%token T_CONNECT_PASS
%token T_USE_VHOST
%token T_VHOST
%token T_SERVER_DESC
%token T_NETNAME
%token T_ADMIN_NAME
%token T_ADMIN_EMAIL
%token T_RECONNECT_TIME
%token T_FLOOD
%token T_FLOOD_TIME
%token T_FLOOD_IGNORE
%token T_LAG_TIME
%token T_STATSERV_HOST
%token T_STATSERV_USER
%token T_ENABLE_SPAM
%token T_ONLYOPERS
%token T_STATSERV_NICK
%token T_WANT_PRIVMSG
%token T_SERVICES_NAME
%token T_LAG_NOTICES
%token T_ENABLE_HTML
%token T_SET_OPER
%token T_SET_DEAF
%token T_HIDE_SERVICES
%token T_HTML_DUMP
%token T_HTML_TIME
%token T_HTML_LINK
%token T_CHANNELS
%token T_URL

%token STRING
%token NUMBER

%type <string> STRING
%type <number> NUMBER

%%

conf:	
	| conf conf_item
	;

conf_item:	server_name
		| server_port
		| connect_to
		| connect_pass
		| use_vhost
		| vhost
		| server_desc
		| netname
		| admin_name
		| admin_email
		| reconnect_time
		| flood
		| flood_time
		| flood_ignore
		| lag_time
		| statserv_host
		| statserv_user
		| statserv_nick
		| enable_spam
		| onlyopers
		| want_privmsg
		| services_name
		| lag_notices
		| enable_html
		| set_oper
		| set_deaf
		| hide_services
		| html_dump
		| html_time
		| html_link
		| channels
		| url
		;

server_name:	T_SERVER_NAME STRING
{
	memcpy(me.name, yylval.string, sizeof(me.name));
};

server_port:	T_SERVER_PORT NUMBER
{
	me.port = yylval.number;
};

connect_to:	T_CONNECT_TO STRING
{
	memcpy(me.uplink, yylval.string, sizeof(me.uplink));
};

connect_pass:	T_CONNECT_PASS STRING
{
	memcpy(me.pass, yylval.string, sizeof(me.pass));
};

use_vhost:	T_USE_VHOST
{
	me.flags |= FLAG_VHOST;
};

vhost:		T_VHOST STRING
{
	memcpy(me.vhost, yylval.string, sizeof(me.vhost));
};

server_desc:	T_SERVER_DESC STRING
{
	memcpy(me.desc, yylval.string, sizeof(me.desc));
};

netname:	T_NETNAME STRING
{
	memcpy(me.netname, yylval.string, sizeof(me.netname));
};

admin_name:	T_ADMIN_NAME STRING
{
	memcpy(me.admin_name, yylval.string, sizeof(me.admin_name));
};

admin_email:	T_ADMIN_EMAIL STRING
{
	memcpy(me.admin_email, yylval.string, sizeof(me.admin_email));
};

reconnect_time:	T_RECONNECT_TIME NUMBER
{
	me.r_time = yylval.number;
};

flood:		T_FLOOD NUMBER
{
	me.flood = yylval.number;
};

flood_time:	T_FLOOD_TIME NUMBER
{
	me.flood_time = yylval.number;
};

flood_ignore:	T_FLOOD_IGNORE
{
	me.flags |= FLAG_FLOOD;
};

lag_time:	T_LAG_TIME NUMBER
{
	me.lag_time = yylval.number;
};

statserv_host:	T_STATSERV_HOST STRING
{
	memcpy(StatServ.host, yylval.string, sizeof(StatServ.host));
};

statserv_user:	T_STATSERV_USER STRING
{
	memcpy(StatServ.user, yylval.string, sizeof(StatServ.user));
};

enable_spam:	T_ENABLE_SPAM
{
	me.flags |= FLAG_SPAM;
};

onlyopers:	T_ONLYOPERS
{
	me.flags |= FLAG_ONLYOPERS;
};

statserv_nick:	T_STATSERV_NICK STRING
{
	memcpy(s_StatServ, yylval.string, sizeof(s_StatServ));
};

want_privmsg:	T_WANT_PRIVMSG
{
	me.flags |= FLAG_PRIVMSG;
};

services_name:	T_SERVICES_NAME STRING
{
	memcpy(me.services_name, yylval.string, sizeof(me.services_name));
};

lag_notices:	T_LAG_NOTICES
{
	me.flags |= FLAG_LAG;
};

enable_html:	T_ENABLE_HTML
{
	me.flags |= FLAG_HTML;
};

set_oper:	T_SET_OPER
{
	me.flags |= FLAG_SETOPER;
};

set_deaf:	T_SET_DEAF
{
	me.flags |= FLAG_SETDEAF;
};

hide_services:	T_HIDE_SERVICES
{
	me.flags |= FLAG_HIDE;
};

html_dump:	T_HTML_DUMP STRING
{
	memcpy(me.html, yylval.string, sizeof(me.html));
};

html_time:	T_HTML_TIME NUMBER
{
	me.html_time = yylval.number;
};

html_link:	T_HTML_LINK STRING
{
	memcpy(me.html_link, yylval.string, sizeof(me.html_link));
};

channels:	T_CHANNELS STRING
{
	memcpy(me.join_chans, yylval.string, sizeof(me.join_chans));
};

url:		T_URL STRING
{
	memcpy(me.url, yylval.string, sizeof(me.url));
};

