/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: statserv.c,v 1.42 2001/04/06 15:38:31 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "help.h"
#include "patchlevel.h"

#ifdef LINUX
/* A kludge in mkpasswd.c, since worthless modern Linux systems require
** _XOPEN_SOURCE to be #define'ed BUT... it screws up the rest of the source
** preventing all str* functions from being defined
*/
extern char *crypt();
#endif

extern const char version_date[], version_time[];
static char errstr[BUFSIZE];

int s_server(char *, int, char **);
int s_map(char *, int, char **);
#ifdef TRACK_VERSION
int s_version_map(char *, int, char **);
#endif
int s_uptime(char *, int, char **);
int s_netstats(char *, int, char **);
int s_daily(char *, int, char **);
int s_tld(char *, int, char **);
int s_tld_map(char *, int, char **);
int s_operlist(char *, int, char **);
int s_login(char *, int, char **);
int s_user(char *, int, char **);
int s_shutdown(char *, int, char **);
int s_restart(char *, int, char **);
int s_stats(char *, int, char **);
int s_logs(char *, int, char **);
#ifdef ENABLE_JUPE
int s_jupe(char *, int, char **);
#endif
int s_join(char *, int, char **);
int s_setpass(char *, int, char **);
#ifdef ENABLE_QUOTE
int s_quote(char *, int, char **);
#endif
int s_cping(char *, int, char **);
int s_cversion(char *, int, char **);
int s_update(char *, int, char **);
int s_url(char *, int, char **);
int s_status(char *, int, char **);
int s_help(char *, int, char **);

static struct MsgTable stat_table[] =
{
	{ "HELP",	0,	s_help },
	{ "SERVER",	0,	s_server },
	{ "MAP",	0,	s_map },
#ifdef TRACK_VERSION
	{ "VERSIONMAP",	0,	s_version_map },
#endif
	{ "UPTIME",	0,	s_uptime },
	{ "NETSTATS",	0,	s_netstats },
	{ "DAILY",	0,	s_daily },
	{ "TLD",	0,	s_tld },
	{ "TLDMAP",	0,	s_tld_map },
	{ "OPERLIST",	0,	s_operlist },
	{ "LOGIN",	0,	s_login },
	{ "USER",	0,	s_user },
	{ "DIE",	0,	s_shutdown },
	{ "SHUTDOWN",	0,	s_shutdown },
	{ "RESTART",	0,	s_restart },
	{ "STATS",	0,	s_stats },
	{ "LOGS",	0,	s_logs },
#ifdef ENABLE_JUPE
	{ "JUPE",	0,	s_jupe },
#endif
	{ "JOIN",	0,	s_join },
	{ "SETPASS",	0,	s_setpass },
#ifdef ENABLE_QUOTE
	{ "QUOTE",	0,	s_quote },
	{ "RAW",	0,	s_quote },
#endif
	{ "\001PING",	0,	s_cping },
	{ "\001VERSION\001",	0,	s_cversion },
	{ "UPDATE",	0,	s_update },
	{ "URL",	0,	s_url },
	{ "STATUS",	0,	s_status },
	{ (char *)0,	0,	(int (*)()) 0 }
};

static struct HelpTable help_table[] = {
        { "SERVER",     ss_server_help,         0 },
        { "MAP",        ss_map_help,            0 },
        { "NETSTATS",   ss_netstats_help,       0 },
        { "DAILY",      ss_daily_help,          0 },
        { "TLD",        ss_tld_help,            0 },
        { "TLDMAP",     ss_tld_map_help,        0 },
        { "OPERLIST",   ss_operlist_help,       0 },
        { "LOGIN",      ss_login_help,          0 },
        { "USER",       ss_user_help,           1 },
        { "SHUTDOWN",   ss_shutdown_help,       1 },
        { "RESTART",    ss_restart_help,        1 },
        { "STATS",      ss_stats_help,          1 },
        { "LOGS",       ss_logs_help,           1 },
        { "SETPASS",    ss_setpass_help,        1 },
        { "JOIN",       ss_join_help,           1 },
        { "UPDATE",     ss_update_help,         1 },
        { "URL",        ss_url_help,            0 },
        { "STATUS",     ss_status_help,         0 },
        { "UPTIME",     ss_uptime_help,         0 },
#ifdef ENABLE_JUPE
        { "JUPE",       ss_jupe_help,           0 },
#endif
#ifdef ENABLE_QUOTE
        { "QUOTE",      ss_quote_help,          1 },
        { "RAW",        ss_quote_help,          1 },
#endif
#ifdef TRACK_VERSION
        { "VERSIONMAP", ss_version_map_help,    0 },
#endif
        { (char *)0,    (const char **)0,       0 }
};


void statserv(char *origin, char *param)
{
	char *parv[MAXPARA];
	int parc = 0;
	User *u = NULL;
	int i;
	int status;
	static char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	/* Clear the error string */
	errstr[0] = '\0';

	strncpy(buf, param, BUFSIZE);

	if (param)
		parc = msgtoken(param, parv);

	if (origin)
		u = finduser(origin);
	else
		return;

        if (!u) {
                log("Unable to finduser %s (statserv)", origin);
                return;
        }

	me.requests++;

        if (flood(u))
                return;

	if (!parc)
		return;

	if (parv[0][0] != '\001')
		log("%s received message from %s: %s", s_StatServ, 
			origin, buf);

        if (IsOnlyOpers() && !IsUIsOper()) {
                privmsg(u->nick, s_StatServ,
                        "This service is only available to IRCops.");
                return;
        }

	/* All the preliminary junk is done, now we have to go through 
	** the tables and call the handler for that command
	*/
        for (i=0; stat_table[i].handler; i++)
        {
                if (!strcasecmp(parv[0], stat_table[i].name))
                {
                    status = stat_table[i].handler(u->nick, parc, parv);
                    if (status == 1)
                        log("Error with statserv command %s: %s", parv[0],
			(errstr) ? errstr : "No error message");
                }
        }
}

/* In all of these functions, parv[0] contains the command, followed
** by the parameters
*/

int s_help(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	int i;

	if (!parv[1]) {
		privmsg_list(u->nick, s_StatServ, ss_help);
		if (u->myuser != NULL)
			privmsg_list(u->nick, s_StatServ, ss_myuser_help);
		return 0;
	}

	for (i = 0; help_table[i].helpmsg; i++)
	{
		if (!strcasecmp(parv[1], help_table[i].name))
                {
			if (help_table[i].myuonly) {
				if (u->myuser)
					privmsg_list(u->nick, s_StatServ,
						help_table[i].helpmsg);
				else
					privmsg(u->nick, s_StatServ,
						"Permission Denied.");
			} else {
				privmsg_list(u->nick, s_StatServ,
					help_table[i].helpmsg);
			}

			return 0;
		}
	}

	privmsg(u->nick, s_StatServ, "Unknown Help Topic: \2%s\2", parv[1]);
	snprintf(errstr, BUFSIZE, "Unknown HELP topic: %s", parv[1]);
	return 1;
}

int s_server(char *origin, int parc, char *parv[])
{
	Server *s = NULL;
	Stats *st = NULL;

	if (!parv[1]) {
		privmsg(origin, s_StatServ,
			"Syntax: /msg %s SERVER <server name>", s_StatServ);
		strcpy(errstr, "Missing parameter to SERVER");
		return 1;
	}

	s = findserver(parv[1]);
	st = findstats(parv[1]);

	if (!st) {
		privmsg(origin, s_StatServ, "No information available on %s",
			parv[1]);
		return 0;
	}

        /* If we want services to be hidden, catch it here */
        if (IsHide() && (!strcasecmp(parv[1], me.services_name)))
        {
                privmsg(origin, s_StatServ,
                    "No information available on %s (hidden server)", parv[1]);
                return 0;
        }

        privmsg(origin, s_StatServ, "Statistics for %s since %s", parv[1],
                sftime(st->starttime));
        if (!s)
                privmsg(origin, s_StatServ, "Server Last Seen:  %s",
                        sftime(st->lastseen));
        if (s)
                privmsg(origin, s_StatServ, "Current Users: %-3ld (%.0f%%)",
			s->users,
                        (float)s->users / (float)me.users * 100);
        privmsg(origin, s_StatServ, "Maximum Users: %-3ld %s", st->maxusers,
                sftime(st->t_maxusers));
        if (s)
                privmsg(origin, s_StatServ, "Current Opers: %d", s->opers);
        privmsg(origin, s_StatServ, "Maximum Opers: %-3d %s", st->maxopers,
                sftime(st->t_maxopers));
        privmsg(origin, s_StatServ, "  IRCop Kills: %d", st->operkills);
        privmsg(origin, s_StatServ, " Server Kills: %d", st->serverkills);
#ifdef LAG_CHECK
        if (s) {
        	privmsg(origin, s_StatServ, "  Lowest Ping: %-3d %s",
			s->lowest_ping, sftime(s->t_lowest_ping));
        	privmsg(origin, s_StatServ, " Highest Ping: %-3d %s",
			s->highest_ping, sftime(s->t_highest_ping));
        	privmsg(origin, s_StatServ, " Current Ping: %d", s->ping);
        }
#endif
        if (st->numsplits >= 1)
        privmsg(origin, s_StatServ, "%s has split from the network %d time%s.",
                st->name, st->numsplits, (st->numsplits == 1) ? "" : "s");
        else
        privmsg(origin, s_StatServ, "%s has never split from the network.",
                st->name);
#ifdef TRACK_VERSION
	if (s) {
	        if (s->version[0])
        	        privmsg(origin, s_StatServ, "Version: %s", s->version);
	        else
        	        privmsg(origin, s_StatServ, "Version Unknown");
	}
#endif
        privmsg(origin, s_StatServ, "End of Statistics.");

	return 0;
}

#ifdef TRACK_VERSION
int s_version_map(char *origin, int parc, char *parv[])
{
        Server *s = NULL;
        int i;

        privmsg(origin, s_StatServ,
                "[\2Server\2]                      [\2Version\2]");
        for (i = 0; i < S_TABLE_SIZE; i++) {
                for (s = serverlist[i]; s; s = s->next) {
                    if (IsHide()) {
                        if (strcasecmp(s->name, me.services_name)) {
                            privmsg(origin, s_StatServ,
                                "%-30s %s", s->name,
				(s->version && s->version[0]) ?  s->version :
				"Unknown Version");
                        }
                    } else {
                        privmsg(origin, s_StatServ, "%-30s %s",
                                s->name, (s->version && s->version[0]) ?
				s->version : "Unknown Version");
                    }
                }
        }
        privmsg(origin, s_StatServ, "End of Version Map.");

	return 0;
}
#endif

int s_map(char *origin, int parc, char *parv[])
{
        Server *s;
        int i;

#ifdef LAG_CHECK
        privmsg(origin, s_StatServ, "Network Map [\2NAME\2] [\2USERS/MAX\2] "
                "[\2OPERS/MAX\2] [\2PERCENT\2] [\2LAG\2]");
#else
        privmsg(origin, s_StatServ, "Network Map [\2NAME\2] [\2USERS/MAX\2] "
                "[\2OPERS/MAX\2] [\2PERCENT\2]");
#endif
        for (i = 0; i < S_TABLE_SIZE; i++) {
                for (s = serverlist[i]; s; s = s->next) {
                    if (IsHide()) {
                        /* meaning if the server does NOT match the services
                        ** server, therefore show it
                        */
                        if (strcasecmp(s->name, me.services_name)) {
                            /* I should also check for juped servers too -
                            ** servers that have services as an uplink
                            */
#ifdef LAG_CHECK
#ifdef LARGE_NET
                            privmsg(origin, s_StatServ,
                                "%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%] %d",
#else
                            privmsg(origin, s_StatServ,
                                "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%] %d",
#endif /* LARGE_NET */
#else
#ifdef LARGE_NET
                            privmsg(origin, s_StatServ,
                                "%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%]",
#else
                            privmsg(origin, s_StatServ,
                                "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%]",
#endif /* LARGE_NET */
#endif /* LAG_CHECK */
                                s->name, s->users, s->stats->maxusers,
                                s->opers, s->stats->maxopers,
                                (float) s->users / (float) me.users * 100
#ifdef LAG_CHECK
                                ,s->ping);
#else
                                );
#endif
                        }
                    } else {
#ifdef LAG_CHECK
#ifdef LARGE_NET
                            privmsg(origin, s_StatServ, "%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%] %d",
#else
                            privmsg(origin, s_StatServ, "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%] %d",
#endif /* LARGE_NET */
#else
#ifdef LARGE_NET
                            privmsg(origin, s_StatServ, "%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%]",
#else
                            privmsg(origin, s_StatServ, "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%]",
#endif /* LARGE_NET */
#endif /* LAG_CHECK */
                                s->name, s->users, s->stats->maxusers,
                                s->opers, s->stats->maxopers,
                                (float) s->users / (float) me.users * 100
#ifdef LAG_CHECK
                                ,s->ping);
#else
                                );
#endif
                    }
                }
        }
#ifdef LARGE_NET
        privmsg(origin, s_StatServ, "%-30s   [%4d/%4d] [%3d/%3d] [100%%] x",
#else
        privmsg(origin, s_StatServ, "%-30s   [%2d/%2d] [%2d/%2d] [100%%] x",
#endif
                 "\2Totals:\2", me.users, me.maxusers, me.opers, me.maxopers);
        privmsg(origin, s_StatServ, "End of Map.");

	return 0;
}

int s_status(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        privmsg(origin, s_StatServ, "Statistics Status:");
        privmsg(origin, s_StatServ, "Reconnect Time: %d", me.r_time);
        privmsg(origin, s_StatServ, "Extreme Lag Notices: %s",
                IsLag() ? "Enabled" : "Disabled");
        if (IsLag())
                privmsg(origin, s_StatServ,
			"Lag notices sent for lag greater than %d seconds",
			me.lag_time);
        privmsg(origin, s_StatServ, "Oper Only: %s",
                IsOnlyOpers() ? "Enabled" : "Disabled");
        privmsg(origin, s_StatServ, "HTML Dump: %s",
                IsHtml() ? "Enabled" : "Disabled");
        if (IsHtml()) {
                privmsg(origin, s_StatServ, "HTML File: %s", me.html);
                privmsg(origin, s_StatServ,
			"HTML File created every %d seconds", me.html_time);
                privmsg(origin, s_StatServ, "URL: %s", me.url);
        }
        privmsg(origin, s_StatServ, "Hide Services: %s",
                (IsHide()) ? "Enabled" : "Disabled");
        if (IsHide())
                privmsg(origin, s_StatServ, "Services Name: %s",
                        me.services_name);
        /* me.name, me.desc, me.uplink all redundant, as they can be seen
        ** from LINKS or from whois s_StatServ
        */
        privmsg(origin, s_StatServ, "Set to oper: %s",
                IsSetOper() ? "Enabled" : "Disabled");

    if (u->myuser) {
        privmsg(origin, s_StatServ, "Spam Service: %s",
                IsSpam() ? "Enabled" : "Disabled");
#ifdef ENABLE_JUPE
        privmsg(origin, s_StatServ, "JUPE: Enabled");
#endif
#ifdef ENABLE_QUOTE
        privmsg(origin, s_StatServ, "QUOTE: Enabled");
#endif
        if (me.flood) {
                privmsg(origin, s_StatServ,
                        "Flood Control: %d requests in %d seconds", me.flood,
                        me.flood_time);
                privmsg(origin, s_StatServ, "Flood Action: %s",
                        IsFlood() ? "IGNORE" : "KILL");
        }
    }

    return 0;
}

int s_uptime(char *origin, int parc, char *parv[])
{
        privmsg(origin, s_StatServ, "Uptime and Memory Statistics:");
        privmsg(origin, s_StatServ, "%s", uptime(me.t_start));
        privmsg(origin, s_StatServ, "Entries in User Hash: %ld", me.users);
        privmsg(origin, s_StatServ, "Entries in Server Hash: %d", me.servers);
        privmsg(origin, s_StatServ, "Statistic Requests: %d", me.requests);
        privmsg(origin, s_StatServ, "User Memory Usage: %d bytes",
                sizeof(User) * me.users);
        privmsg(origin, s_StatServ, "Server Memory Usage: %d bytes",
                sizeof(Server) * me.servers);
        privmsg(origin, s_StatServ, "End of Information.");

	return 0;
}

int s_netstats(char *origin, int parc, char *parv[])
{
        privmsg(origin, s_StatServ, "Network Statistics:");
        privmsg(origin, s_StatServ, "   Current Users: %ld", me.users);
        privmsg(origin, s_StatServ, "   Maximum Users: %-2ld %s", me.maxusers,
                sftime(me.t_maxusers));
        privmsg(origin, s_StatServ, "   Current Opers: %d", me.opers);
        privmsg(origin, s_StatServ, "   Maximum Opers: %-2d %s", me.maxopers,
                sftime(me.t_maxopers));
        privmsg(origin, s_StatServ, "  Users Set Away: %-2d", me.away);
        privmsg(origin, s_StatServ, " Current Servers: %d", me.servers);
        privmsg(origin, s_StatServ, " Maximum Servers: %-2d %s", me.maxservers,
                sftime(me.t_maxservers));
        privmsg(origin, s_StatServ, "End of Information.");

	return 0;
}

int s_daily(char *origin, int parc, char *parv[])
{
        privmsg(origin, s_StatServ, "Daily Network Statistics:");
        privmsg(origin, s_StatServ, " ");
        privmsg(origin, s_StatServ, "Maximum Servers: %-2d %s", daily.servers,
                sftime(daily.t_servers));
        privmsg(origin, s_StatServ, "  Maximum Users: %-2d %s", daily.users,
                sftime(daily.t_users));
        privmsg(origin, s_StatServ, "  Maximum Opers: %-2d %s", daily.opers,
                sftime(daily.t_opers));
        privmsg(origin, s_StatServ, "All daily statistics are reset at midnight.");
        privmsg(origin, s_StatServ, "End of Information.");

	return 0;
}

int s_tld(char *origin, int parc, char *parv[])
{
        TLD *tmp;
	char *tld = parv[1];

        if (!tld) {
                privmsg(origin, s_StatServ, "Syntax: /msg %s TLD <tld>", 
			s_StatServ);
                privmsg(origin, s_StatServ, 
			"For additional help, /msg %s HELP", s_StatServ);
		strcpy(errstr, "Missing parameter in TLD");
                return 1;
        }

        if (*tld == '*')
                tld++;
        if (*tld == '.')
                tld++;

	/* Catch it when it's just * or . */
	if (!(*tld)) {
		privmsg(origin, s_StatServ, "Syntax: /msg %s TLD <tld>",
			s_StatServ);
		privmsg(origin, s_StatServ,
			"For additional help, /msg %s HELP", s_StatServ);
		strcpy(errstr, "Missing parameter in TLD");
		return 1;
	}

        tmp = findtld(tld);

        if (!tmp)
                privmsg(origin, s_StatServ,
			"Top Level Domain \2%s\2 does not exist.", tld);
        else
                privmsg(origin, s_StatServ, "%s", tmp->country);

	return 0;
}

int s_tld_map(char *origin, int parc, char *parv[])
{
        TLD *t;

        privmsg(origin, s_StatServ, "Top Level Domain Statistics:");
        for (t = tldhead; t; t = t->next) {
                if (t->users != 0)
                privmsg(origin, s_StatServ, "%9s \2%3d\2 (%2.0f%%) -> %s", 
			t->tld, t->users,
			(float)t->users / (float)me.users * 100, t->country);
        }
        privmsg(origin, s_StatServ, "End of Information.");

	return 0;
}

int s_operlist(char *origin, int parc, char *parv[])
{
        int i, j = 0;
        int away = 0;
        User *u;
	char *server = parv[2];

        if (parv[1] && !strcasecmp(parv[1], "NOAWAY")) {
                away = 1;
		/* hmmmmm */
                parv[1] = NULL;
        }
        if (!away && parv[1] && strchr(parv[1], '.'))
                server = parv[1];

        privmsg(origin, s_StatServ, "On-Line IRCops:");

        for (i = 0; i < U_TABLE_SIZE; i++) {
                for (u = userlist[i]; u; u = u->next) {
                        if (away && IsUIsAway())
                                continue;
                        if (IsHide() &&
                            !strcasecmp(u->server->name, me.services_name))
                                continue;
                        if (!IsUIsOper())
                                continue;
                        if (!server) {
                                if (!IsUIsOper()) continue;
                                j++;
                                privmsg(origin, s_StatServ, 
#ifdef OPER_MODES
					"[%2d] [%c] %-15s %s", j, opermode(u),
					u->nick,
#else
					"[%2d] %-15s %s",j, u->nick,
#endif
                                        u->server->name);
                                continue;
                        } else {
                                if (strcasecmp(server, u->server->name))
					continue;
                                if (!IsUIsOper()) continue;
                                j++;
				privmsg(origin, s_StatServ,
#ifdef OPER_MODES
					"[%2d] [%c] %-15s %s", j, opermode(u),
#else
					"[%2d] %-15s %s", j,
#endif
					u->nick, u->server->name);
                                continue;
                        }
                }
        }
        privmsg(origin, s_StatServ, "End of Listing.");

	return 0;
}

int s_login(char *origin, int parc, char *parv[])
{
        MyUser *myu;
#if defined(CRYPT) || defined(MD5)
        char * crpass;
#endif
	char *user=parv[1];
	char *pass=parv[2];
	User *u = finduser(origin);

        if (!user || !pass) {
                privmsg(origin, s_StatServ,
			"Syntax: /msg %s LOGIN \37username\37 \37password\37",
			s_StatServ);
		strcpy(errstr, "Missing parameters in LOGIN");
                return 1;
        }

	if (!u) {
		snprintf(errstr, BUFSIZE, "Unknown user: %s", origin);
		return 1;
	}

        myu = findmyuser(user);
        if (!myu) {
                privmsg(origin, s_StatServ, "Unknown User.");
                log("Failed login attempt (unknown user (%s))", user);
		/* This isn't an error in the code, only to the user */
                return 0;
        }
#if defined(CRYPT) || defined(MD5)
        /* Passing in the whole password is fine, no need to rip of the
        ** salt.  This actually will support md5, but has not been tested
        */
        crpass = crypt(pass, myu->password);
        if (!strcasecmp(myu->username, user) && 
            !strcmp(myu->password, crpass)) {
#else
        if (!strcasecmp(myu->username, user) && 
            !strcmp(myu->password, pass)) {
#endif /* CRYPT || MD5 */
                log("%s [%s](%s@%s) now walks among the gods.", u->nick,
                        myu->username, u->username, u->hostname);
                u->myuser = myu;
                privmsg(u->nick, s_StatServ, "Permission Granted.");
                privmsg(u->nick, s_StatServ, "You now walk among the gods.");
                myu->lastseen = time(NULL);
                myu->flags |= MFLAG_ISON;
                /* We keep the CURRENT person who LOGIN'ed through that
                ** access entry pointed to by the u pointer.  This is used
                ** to obtain information on the user who logged in, ie.
                ** in STATS p
                */
                myu->u = u;
                return 0;
        } else {
                log("%s [%s](%s) failed login.", u->nick, myu->username,
			u->hostname);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* This isn't a code error either */
		return 0;
        }

	/* NOT REACHED */
	return 0;
}

int s_user(char *origin, int parc, char *parv[])
{
        int a = 0;
	User *u = finduser(origin);
	char *what = parv[1];
	char *user = parv[2];
	char *pass = parv[3];

        if (!what) {
                privmsg(u->nick, s_StatServ,
			"Syntax: /msg %s USER [ADD|DEL|LIST]", s_StatServ);
                privmsg(u->nick, s_StatServ,
			"For additional help, /msg %s HELP", s_StatServ);
		strcpy(errstr, "Missing parameter in USER");
                return 1;
        }

        if (!u->myuser) {
                log("Denied access (USER) to %s [%s].", u->nick, u->hostname);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!strcasecmp(what, "LIST"))
                a = 0;
        else if (!strcasecmp(what, "ADD"))
                a = 1;
        else if (!strcasecmp(what, "DEL"))
                a = 2;
        else {
                privmsg(u->nick, s_StatServ, "Unknown Flag: %s", what);
		/* Not REALLY a code error...but... */
		snprintf(errstr, BUFSIZE, "Unknown flag: %s in USER", what);
                return 1;
        }

        switch (a) {
                case 0: {
                        MyUser *myu;
                        privmsg(u->nick, s_StatServ, "UserList:");
                        for (myu = myuhead; myu; myu = myu->next) {
                                privmsg(u->nick, s_StatServ, 
					"%-20s (%s) -> %s", myu->username,
                                        (!IsMIsOn()) ? 
						"offline" : " online",
                                        (myu->lastseen == 0) ? "Never Seen" :
						sftime(myu->lastseen));
                        }
                        privmsg(u->nick, s_StatServ, "End of List.");
		}
                case 1: {
                        MyUser *myu;
#if defined(CRYPT) || defined(MD5)
                        char * crpass;
                        char * salt;
#endif /* CRYPT || MD5 */
                        if (!user || !pass) {
                                privmsg(u->nick, s_StatServ,
					"Syntax: /msg %s USER ADD "
                                        "<username> <password>", s_StatServ);
				strcpy(errstr, "Missing parameter in USER");
                                return 1;
                        }
                        myu = smalloc(sizeof(MyUser));
                        myu->username = strdup(user);
#if defined(CRYPT) || defined(MD5)
                        salt = make_salt();
                        crpass = crypt(pass, salt);
                        myu->password = strdup(crpass);
                        /* if I free(crpass), I'll get runtime errors - it's
			** static inside of crypt()
			*/
#else
                        myu->password = strdup(pass);
#endif /* CRYPT */
                        myu->lastseen = time(NULL);
                        if (!myuhead) {
                                myuhead = myu;
                                myuhead->next = NULL;
                        } else {
                                myu->next = myuhead;
                                myuhead = myu;
                        }
                        privmsg(u->nick, s_StatServ,
				"Added user %s with password %s", user, pass);
                        log("%s [%s](%s) added user %s", u->nick,
				u->myuser->username, u->hostname,
				myu->username);
                        SaveMyUsers();
			return 0;
                }
                case 2: {
                        if (!user) {
                                privmsg(u->nick, s_StatServ,
					"Syntax: /msg %s USER DEL <username>",
                                        s_StatServ);
				strcpy(errstr, "Missing parameter in USER");
				return 1;
                        }
                        if ((findmyuser(user)) == NULL) {
                                privmsg(u->nick, s_StatServ,
					"Unable to find user %s", user);
				snprintf(errstr, BUFSIZE,
					"Unknown myuser: %s", user);
				return 1;
                        }
                        DeleteMyUser(user);
                        SaveMyUsers();
                        privmsg(u->nick, s_StatServ, "Deleted user %s", user);
                        log("%s [%s](%s) deleted user %s", u->nick,
				u->myuser->username, u->hostname, user);
			return 0;
                }
        }

	/* NOT REACHED */
	return 0;
}

int s_shutdown(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char comm[BUFSIZE];
	int i;

        if (!u->myuser) {
                log("Access Denied (SHUTDOWN) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (parv[1])
                strncpy(comm, parv[1], BUFSIZE);
        for (i = 2; i < parc; i++)
                snprintf(comm, BUFSIZE, "%s %s", comm, parv[i]);

        /* Yes, we are very nasty like this. */
        globops(me.name, "%s requested SHUTDOWN: %s", u->nick,
                (comm) ? comm : "No Reason");
        SendQuit(u, 0, comm);
        sleep(1);
        close(servsock);
        SaveStats();
        log("%s [%s](%s) requested SHUTDOWN: %s", u->nick, u->myuser->username,
                u->hostname, (comm) ? comm : "No Reason");
        exit(0);

	/* NOT REACHED */
	return 0;
}

int s_restart(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char comm[BUFSIZE];
	int i;

        if (!u->myuser) {
                log("Access Denied (RESTART) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

	if (parv[1])
		strncpy(comm, parv[1], BUFSIZE);
	for (i = 2; i < parc; i++)
		snprintf(comm, BUFSIZE, "%s %s", comm, parv[i]);

        /* Yes, we are very nasty like this. */
        globops(me.name, "%s requested RESTART: %s", u->nick,
                (comm) ? comm : "No Reason");
        SendQuit(u, 1, comm);
        sleep(1);
        close(servsock);
        SaveStats();
        log("%s [%s](%s) requested RESTART: %s", u->nick, u->myuser->username,
                u->hostname, (comm) ? comm : "No Reason");
        execv(SPATH, NULL);

	/* NOT REACHED */
	return 0;
}

int s_stats(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *cmd = parv[1];
	char *arg = parv[2];
	char *arg2 = parv[3];
        Stats *st;

        if (!u->myuser) {
                log("Access Denied (STATS) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!cmd) {
                privmsg(u->nick, s_StatServ,
			"Syntax: /msg %s STATS [DEL|LIST|COPY]",
                        s_StatServ);
                privmsg(u->nick, s_StatServ,
			"For additional help, /msg %s HELP", s_StatServ);
		strcpy(errstr, "Missing parameter in STATS");
                return 1;
        }
        if (!strcasecmp(cmd, "LIST")) {
                int i = 1;
                privmsg(u->nick, s_StatServ, "Statistics Database:");
                for (st = shead; st; st = st->next) {
                        privmsg(u->nick, s_StatServ, "[%-2d] %s", i, st->name);
                        i++;
                }
                privmsg(u->nick, s_StatServ, "End of List.");
                log("%s requested STATS LIST.", u->nick);
        } else if (!strcasecmp(cmd, "DEL")) {
                Stats *m = NULL;
                if (!arg) {
                        privmsg(u->nick, s_StatServ,
				"Syntax: /msg %s STATS DEL <name>",
				s_StatServ);
                        privmsg(u->nick, s_StatServ,
				"For additonal help, /msg %s HELP",
				s_StatServ);
			strcpy(errstr, "Missing parameter in STATS");
                        return 1;
                }
                st = findstats(arg);
                if (!st) {
                        privmsg(u->nick, s_StatServ,
				"%s is not in the database!", arg);
			snprintf(errstr, BUFSIZE, "Unknown server: %s", arg);
                        return 1;
                }
                for (st = shead; st; st = st->next) {
                        if (!strcasecmp(arg, st->name))
                                break;
                        m = st;
                }
                if (m)
                        m->next = st->next;
                else
                        shead = st->next;
                free(st);
                privmsg(u->nick, s_StatServ, "Removed %s from the database.",
			arg);
                log("%s requested STATS DEL %s", u->nick, arg);
        } else if (!strcasecmp(cmd, "COPY")) {
                Server *s;

                if (!arg || !arg2) {
                        privmsg(u->nick, s_StatServ,
				"Syntax: /msg %s STATS COPY <name> <newname>",
				s_StatServ);
			strcpy(errstr, "Missing parameter in STATS");
                        return 1;
                }
                st = findstats(arg2);
                if (st)
                        free(st);

                st = findstats(arg);
                if (!st) {
                        privmsg(u->nick, s_StatServ,
				"No entry in the database for %s", arg);
			snprintf(errstr, BUFSIZE, "Unknown stats entry: %s",
				arg);
                        return 1;
                }
                s = findserver(arg);
                if (s) {
                        privmsg(u->nick, s_StatServ, "Server %s is online!",
				arg);
			snprintf(errstr, BUFSIZE, "Server %s is online", arg);
                        return 1;
                }
                s = NULL;
                memcpy(st->name, arg2, sizeof(st->name));
                s = findserver(st->name);
                if (s)
                        s->stats = st;
                privmsg(u->nick, s_StatServ,
			"Moved database entry for %s to %s", arg, arg2);
                log("%s requested STATS COPY %s -> %s", u->nick, arg, arg2);
        } else {
                privmsg(u->nick, s_StatServ, "Invalid Argument.");
                privmsg(u->nick, s_StatServ, "For help, /msg %s HELP",
			s_StatServ);
		strcpy(errstr, "Invalid argument in STATS");
		return 1;
        }

	return 0;
}

/* This function needs to go */
int s_logs(char *origin, int parc, char *parv[])
{

        FILE *fp;
        char buf[512];

	User *u = finduser(origin);

        if (!u->myuser) {
                log("Access Denied (LOGS) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        log("%s requested LOGS", u->nick);
	if (IsDebug()) {
		privmsg(u->nick, s_StatServ,
			"This command is disabled while in DEBUG.");
	} else {
        	fp = fopen(LOGFILE, "r");
        	if (!fp) {
                	privmsg(u->nick, s_StatServ,
				"Unable to open %s", LOGFILE);
                	return 0;
        }
        while (fgets(buf, sizeof(buf), fp)) {
                buf[strlen(buf)] = '\0';
                privmsg(u->nick, s_StatServ, "%s", buf);
        }
        fclose(fp);
}

	return 0;
}

#ifdef ENABLE_QUOTE
int s_quote(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char command[BUFSIZE];
	int i;

        if (!u->myuser) {
                log("Access Denied (QUOTE) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not an error in the code */
                return 0;
        }

        if (parv[1])
                strncpy(command, parv[1], BUFSIZE);
        for (i = 2; i < parc; i++)
                snprintf(command, BUFSIZE, "%s %s", command, parv[i]);

        if (!command) {
                privmsg(u->nick, s_StatServ, "Please enter a valid command");
		strcpy(errstr, "NULL command attempt in QUOTE");
                return 1;
        }

        sts(command);
        log("%s!%s@%s made me execute %s", u->nick, u->username,
                u->hostname, command);

	return 0;
}
#endif

#ifdef ENABLE_JUPE
int s_jupe(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *server = parv[1];
        Server *s;
        /* NOTE:  should I prevent the main services server from
        ** being juped here?
        ** Also prevent my uplink from being JUPED.
        ** JUPE is a touchy feature anyway...
        **
        ** ALSO.... We should maybe make JUPE's more perminant...
        ** Or just remove JUPE functionality completely and use
        ** an updated jupe.c/jupe.pl or main services.
        */

#ifdef ALL_OPER_JUPE
        if (!IsUIsOper()) {
#else
        if (!IsUIsOper() || !u->myuser) {
#endif
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }
#ifdef ALL_OPER_JUPE
        if (IsUIsOper()) {
#else
        if (IsUIsOper() && u->myuser) {
#endif
                if (!server) {
                        privmsg(u->nick, s_StatServ,
                                "Syntax: /msg %s JUPE <server>", s_StatServ);
			strcpy(errstr, "Missing parameter in JUPE");
                        return 1;
                }
                s = findserver(server);
                if (s) {
                        sts(":%s SQUIT %s :JUPED by %s", s_StatServ, server,
                                u->nick);
                        globops("SERVER %s JUPED by %s", server, u->nick);
                        log("SERVER %s SQUIT'ed by JUPE", server);
                }
                sts(":%s SERVER %s 1 :[Jupitered] by %s", me.name, server,
                        u->nick);
                log("%s!%s@%s jupitered %s", u->nick, u->username,
                        u->hostname, server);
        } else {
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

	return 0;
}
#endif

int s_join(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *chan = parv[1];

        if (!u->myuser) {
                log("Access Denied (JOIN) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (chan) {
                if (chan[0] != '#') {
                        privmsg(u->nick, s_StatServ,
                                "Please enter a valid channel");
			snprintf(errstr, BUFSIZE, "Invalid channel: %s", chan);
                        return 1;
                }

#if defined(IRCD_DAL) || defined(IRCD_DALNQ)
                        sts(":%s JOIN %s", s_StatServ, chan);
#else
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
                        sts(":%s SJOIN %ld %s + :@%s", me.name, time(NULL),
                                chan, s_StatServ);
#else
#if defined(IRCD_MLN)
                        sts(":%s JOIN %s", s_StatServ, chan);
                        sts(":%s MODE %s +o %s %ld", me.name, chan,
                                s_StatServ, time(NULL));
#else
#if defined(IRCD_2_10)
                        sts(":%s NJOIN %s :@%s", me.name, chan, s_StatServ);
#endif
#endif
#endif
#endif

                log("%s!%s@%s made me join %s", u->nick, u->username,
                        u->hostname, chan);

                return 0;
        } else {
                privmsg(u->nick, s_StatServ, "Please enter a valid channel");
		strcpy(errstr, "NULL channel attempt in JOIN");
                return 1;
        }
}

int s_setpass(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *pass = parv[1];
#if defined(CRYPT) || defined(MD5)
        char * crpass;
        char * salt;
#endif /* CRYPT || MD5 */

        if (!u->myuser) {
                log("Access Denied (SETPASS) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!pass) {
                privmsg(u->nick, s_StatServ, "Please enter a valid password");
		strcpy(errstr, "NULL password attempt in SETPASS");
                return 1;
        }

        /* First I'll free the memory the current password is using */
        free(u->myuser->password);

#if defined(CRYPT) || defined(MD5)
        salt = make_salt();
        crpass = crypt(pass, salt);
        u->myuser->password = strdup(crpass);
        /* if I free(crpass), I'll get runtime errors */
#else
        u->myuser->password = strdup(pass);
#endif /* CRYPT */

        privmsg(u->nick, s_StatServ, "Password changed to %s", pass);
        /* No I will NOT log the password! */
        log("%s!%s@%s changed his/her password", u->nick, u->username,
                u->hostname);
	SaveMyUsers();

	return 0;
}

int s_cping(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *command = parv[1];

        if (!command)
                return 1;
        sts(":%s NOTICE %s :\001PING %s", s_StatServ, u->nick, command);
        log("CTCP PING from %s!%s@%s", u->nick, u->username, u->hostname);

	return 0;
}

int s_cversion(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        sts(":%s NOTICE %s :\001VERSION %s(%s) %s %s -> %s %s\001", s_StatServ,
             u->nick, PATCHLEVEL, SERIALNUM, me.name, VersionFlags(),
	     version_date, version_time);
        log("CTCP VERSION from %s!%s@%s", u->nick, u->username, u->hostname);

	return 0;
}

int s_update(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        if (!u->myuser) {
                log("Access Denied (UPDATE) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!IsHtml()) {
                privmsg(u->nick, s_StatServ, "HTML Update Disabled");
                return 0;
        }

        DumpHTML();
        privmsg(u->nick, s_StatServ, "HTML Update Forced");

        log("%s!%s@%s forces an HTML Update", u->nick, u->username,
		u->hostname);
	return 0;
}

int s_url(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        if (!IsHtml()) {
                privmsg(u->nick, s_StatServ, "HTML Dump Disabled");
                return 0;
        }

        privmsg(u->nick, s_StatServ, "HTML Status URL: %s", me.url);

        log("%s!%s@%s Looks up URL", u->nick, u->username, u->hostname);

	return 0;
}

/* process messages sent to the spam detecter */
void spam(char *origin, char *line)
{
	User *u = finduser(origin);

	if (!u) {
		log("Unable to find user %s (spam)", origin);
		return;
	}

	if (flood(u))
		return;

	if (IsUIsOper())
		return;

	globops(me.name, "Possible Mass Message -\2(%s!%s@%s)\2- %s", u->nick,
		u->username, u->hostname, line);
	log("Possible Mass Message -(%s!%s@%s)- %s", u->nick, u->username,
		u->hostname, line);
}

